@@ -1,4 +1,4 @@
-# $Id: wcloud.py 1080 2014-10-22 13:43:08Z mwall $
+# $Id: wcloud.py 1343 2015-07-24 23:49:19Z mwall $
 # Copyright 2014 Matthew Wall
 
 """
@@ -10,6 +10,19 @@ Based on weathercloud API documentation v0.5 as of 15oct2014.
 
 The preferred upload frequency (post_interval) is one record every 10 minutes.
 
+These are the possible states for sensor values:
+- sensor exists and returns valid value
+- sensor exists and returns invalid value that passes StdQC
+- sensor exists and returns None (e.g., windDir when windSpeed is zero)
+- sensor exists but is not working
+- sensor does not exist
+
+Regarding None/NULL values, the folks at weathercloud say the following:
+
+"In order to fix this issue, WeeWX should send our error code (-32768) instead
+of omitting that variable in the frame. This way we know that the device is
+able to measure the variable but that it's not currently working."
+
 Minimal Configuration:
 
 [StdRESTful]
@@ -19,6 +32,7 @@ Minimal Configuration:
 """
 
 import Queue
+import re
 import sys
 import syslog
 import time
@@ -29,9 +43,13 @@ import weewx
 import weewx.restx
 import weewx.units
 import weewx.wxformulas
-from weeutil.weeutil import to_bool
+from weeutil.weeutil import to_bool, accumulateLeaves
 
-VERSION = "0.5"
+VERSION = "0.8"
+
+if weewx.__version__ < "3":
+    raise weewx.UnsupportedFeature("weewx 3 is required, found %s" %
+                                   weewx.__version__)
 
 def logmsg(level, msg):
     syslog.syslog(level, 'restx: WeatherCloud: %s' % msg)
@@ -47,6 +65,8 @@ def logerr(msg):
 
 # weewx uses a status of 1 to indicate failure, wcloud uses 0
 def _invert(x):
+    if x is None:
+        return None
     if x == 0:
         return 1
     return 0
@@ -56,11 +76,10 @@ def _convert_windspeed(v, from_unit_system):
     if from_unit_system is None:
         return None
     if from_unit_system != weewx.METRICWX:
-        (from_unit, _) = weewx.units.getStandardUnitType(from_unit_system,
-                                                         'windSpeed')
+        (from_unit, _) = weewx.units.getStandardUnitType(
+            from_unit_system, 'windSpeed')
         from_t = (v, from_unit, 'group_speed')
-        to_t = weewx.units.convert(from_t, 'meter_per_second')
-        v = to_t.value
+        v = weewx.units.convert(from_t, 'meter_per_second')[0]
     return v
 
 # FIXME: this formula is suspect
@@ -73,45 +92,33 @@ def _calc_thw(heatindex_C, windspeed_mps):
     thw_C = (thw_F - 32) * 5 / 9
     return thw_C
 
-def _get_windavg(archive, ts, interval=600):
+def _get_windavg(dbm, ts, interval=600):
     sts = ts - interval
-    val = archive.getSql("SELECT AVG(windSpeed) FROM archive "
-                         "WHERE dateTime>? AND dateTime<=?",
-                         (sts, ts))
-    if val is None:
-        return None
-    return val[0]
+    val = dbm.getSql("SELECT AVG(windSpeed) FROM %s "
+                     "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
+                     (sts, ts))
+    return val[0] if val is not None else None
 
 # weathercloud wants "10-min maximum gust of wind".  some hardware reports
 # a wind gust, others do not, so try to deal with both.
-def _get_windhi(archive, ts, interval=600):
+def _get_windhi(dbm, ts, interval=600):
     sts = ts - interval
-    val = archive.getSql("""SELECT
+    val = dbm.getSql("""SELECT
  MAX(CASE WHEN windSpeed >= windGust THEN windSpeed ELSE windGust END)
- FROM archive
- WHERE dateTime>? AND dateTime<=?""",
-                         (sts, ts))
-    if val is None:
-        return None
-    return val[0]
+ FROM %s
+ WHERE dateTime>? AND dateTime<=?""" % dbm.table_name, (sts, ts))
+    return val[0] if val is not None else None
 
-def _get_winddiravg(archive, ts, interval=600):
+def _get_winddiravg(dbm, ts, interval=600):
     sts = ts - interval
-    val = archive.getSql("SELECT AVG(windDir) FROM archive "
-                         "WHERE dateTime>? AND dateTime<=?",
-                         (sts, ts))
-    if val is None:
-        return None
-    return val[0]
+    val = dbm.getSql("SELECT AVG(windDir) FROM %s "
+                     "WHERE dateTime>? AND dateTime<=?" %
+                     dbm.table_name, (sts, ts))
+    return val[0] if val is not None else None
 
 class WeatherCloud(weewx.restx.StdRESTbase):
-    """Upload using the WeatherCloud protocol."""
-
     def __init__(self, engine, config_dict):
-        """Initialize for upload to WeatherCloud.  This service supports the
-        standard restful options plus the following:
-
-        Required parameters:
+        """This service recognizes standard restful options plus the following:
 
         id: WeatherCloud identifier
 
@@ -120,13 +127,15 @@ class WeatherCloud(weewx.restx.StdRESTbase):
         super(WeatherCloud, self).__init__(engine, config_dict)
         loginf("service version is %s" % VERSION)
         try:
-            site_dict = weewx.restx.get_dict(config_dict, 'WeatherCloud')
+            site_dict = config_dict['StdRESTful']['WeatherCloud']
+            site_dict = accumulateLeaves(site_dict, max_level=1)
             site_dict['id']
             site_dict['key']
         except KeyError, e:
             logerr("Data will not be posted: Missing option %s" % e)
             return
-        site_dict.setdefault('database_dict', config_dict['Databases'][config_dict['StdArchive']['archive_database']])
+        site_dict['manager_dict'] = weewx.manager.get_manager_dict(
+            config_dict['DataBindings'], config_dict['Databases'], 'wx_binding')
 
         self.archive_queue = Queue.Queue()
         self.archive_thread = WeatherCloudThread(self.archive_queue, **site_dict)
@@ -204,14 +213,14 @@ class WeatherCloudThread(weewx.restx.RESTThread):
 #                 'forecasticon': ('??',       '%.0f', 1.0),
                  }
 
-    def __init__(self, queue, id, key, database_dict,
+    def __init__(self, queue, id, key, manager_dict,
                  server_url=_SERVER_URL, skip_upload=False,
                  post_interval=600, max_backlog=sys.maxint, stale=None,
                  log_success=True, log_failure=True,
                  timeout=60, max_tries=3, retry_wait=5):
         super(WeatherCloudThread, self).__init__(queue,
                                                protocol_name='WeatherCloud',
-                                               database_dict=database_dict,
+                                               manager_dict=manager_dict,
                                                post_interval=post_interval,
                                                max_backlog=max_backlog,
                                                stale=stale,
@@ -224,37 +233,38 @@ class WeatherCloudThread(weewx.restx.RESTThread):
         self.key = key
         self.server_url = server_url
         self.skip_upload = to_bool(skip_upload)
-        self.archive_units = None
 
-    def process_record(self, record, archive):
-        r = self.get_record(record, archive)
+    def process_record(self, record, dbm):
+        r = self.get_record(record, dbm)
         url = self.get_url(r)
         if self.skip_upload:
-            logdbg("skipping upload")
+            loginf("skipping upload")
             return
         req = urllib2.Request(url)
         req.add_header("User-Agent", "weewx/%s" % weewx.__version__)
         self.post_with_retries(req)
 
     # calculate derived quantities and other values needed by wcloud
-    def get_record(self, record, archive):
-        if self.archive_units is None:
-            u = archive.getSql("SELECT usUnits FROM archive LIMIT 1")
-            if u is not None:
-                self.archive_units = u[0]
-        rec = super(WeatherCloudThread, self).get_record(record, archive)
+    def get_record(self, record, dbm):
+        rec = super(WeatherCloudThread, self).get_record(record, dbm)
 
         # put everything into units required by weathercloud
         rec = weewx.units.to_METRICWX(rec)
 
         # calculate additional quantities
-        rec['windavg'] = _get_windavg(archive, record['dateTime'])
-        rec['windhi'] = _get_windhi(archive, record['dateTime'])
-        rec['winddiravg'] = _get_winddiravg(archive, record['dateTime'])
+        rec['windavg'] = _get_windavg(dbm, record['dateTime'])
+        rec['windhi'] = _get_windhi(dbm, record['dateTime'])
+        rec['winddiravg'] = _get_winddiravg(dbm, record['dateTime'])
+
+        # ensure wind direction is in [0,359]
+        if rec['windDir'] > 359:
+            rec['windDir'] -= 360
+        if rec['winddiravg'] > 359:
+            rec['winddiravg'] -= 360
 
         # these observations are non-standard, so do unit conversions directly
-        rec['windavg'] = _convert_windspeed(rec['windavg'], self.archive_units)
-        rec['windhi'] = _convert_windspeed(rec['windhi'], self.archive_units)
+        rec['windavg'] = _convert_windspeed(rec['windavg'], record['usUnits'])
+        rec['windhi'] = _convert_windspeed(rec['windhi'], record['usUnits'])
 
         if 'inTemp' in rec and 'inHumidity' in rec:
             rec['inheatindex'] = weewx.wxformulas.heatindexC(
@@ -284,11 +294,13 @@ class WeatherCloudThread(weewx.restx.RESTThread):
         values['key'] = self.key
         time_tt = time.gmtime(record['dateTime'])
         values['time'] = time.strftime("%H%M", time_tt) # assumes leading zeros
+        values['date'] = time.strftime("%Y%m%d", time_tt)
         for key in self._DATA_MAP:
             rkey = self._DATA_MAP[key][0]
             if record.has_key(rkey) and record[rkey] is not None:
                 v = record[rkey] * self._DATA_MAP[key][2]
                 values[key] = self._DATA_MAP[key][1] % v
         url = self.server_url + '?' + urllib.urlencode(values)
-        logdbg('url: %s' % url)
+        if weewx.debug >= 2:
+            logdbg('url: %s' % re.sub(r"key=[^\&]*", "key=XXX", url))
         return url
